------------------------------------------------
-- Creating a Table with an ORDImage Column
------------------------------------------------
connect scott/tiger
create table image_table (id number primary key, image ordsys.ordimage);


------------------------------------------------
-- Importing Images into the Database
------------------------------------------------
connect / as sysdba
create or replace directory imagedir as '/home/alamb/quickstart';
-- For windows:
--create or replace directory imagedir as 'c:\quickstart';
grant read on directory imagedir to scott;

connect scott/tiger
-- imports the file named <filename> in IMAGEDIR into image_table in a new 
-- row. The new row has the id identifier set to <dest_id>
create or replace procedure image_import(dest_id number, filename varchar2) is
  img ordsys.ordimage;
  ctx raw(64) := null;
begin
   delete from image_table where id = dest_id;
   insert into image_table (id, image) 
     values (dest_id, ordsys.ordimage.init()) 
     returning image into img;

   img.importFrom(ctx, 'file', 'IMAGEDIR', filename);
   update image_table set image=img where id=dest_id;
end;
/

call image_import(1,'flowers.jpg');
call image_import(2,'goats.gif');

------------------------------------------------
-- Selecting and Viewing Image Properties
-- (Modified to generate nice output)
------------------------------------------------
connect scott/tiger
select id            "id", 
       t.image.getheight() "height",
       t.image.getwidth()  "width"
from image_table t;

-- the column statements format the output nicely
column "fileformat"    format A30;
column "compression"   format A30;
column "contentformat" format A30;

select id "id", 
       t.image.getfileformat() "fileformat",
       t.image.getcompressionformat() "compression" 
from image_table t;


select id "id",
       t.image.getcontentformat() "contentformat", 
       t.image.getcontentlength() "length"
from image_table t;


------------------------------------------------
-- Creating Thumbnails and Changing Formats
------------------------------------------------
connect scott/tiger
-- Generates a new ORDImage in row with id=<dest_id> based on <verb> and the ORDImage 
-- in the row with id=<source_id>.
-- NOTE: assumes <source_id> not equal to <dest_id>
create or replace procedure image_processCopy(source_id number, dest_id number, verb varchar2) is
    imgSrc    ordsys.ordimage;
    imgDst    ordsys.ordimage;
begin
  delete from image_table where id = dest_id;
  insert into image_table (id, image) 
    values (dest_id, ordsys.ordimage.init());
  select image into imgSrc from image_table where id = source_id;
  select image into imgDst from image_table where id = dest_id for update;
  imgSrc.processCopy(verb, imgDst);
  update image_table set image = imgDst where id = dest_id;
end;
/

-- Scale flowers.jpg to 10% into row with id=3
call image_processcopy(1,3,'scale=.1');

-- convert goats.gif to grayscale jpeg thumbnail into row with id=4
call image_processcopy(2,4,'fileformat=jfif contentformat=8bitgray maxscale=100 100');

-- look at our handiwork
column t.image.getfileformat() format A20;
select id, t.image.getWidth(), t.image.getHeight(), t.image.getFileFormat() 
       from image_table t;

-- convert goats.gif to grayscale png thumnbail
call image_processcopy(2,5,'fileformat=pngf contentformat=8bitgray maxscale=100 100');

------------------------------------------------
-- Exporting Images with ORDImage.export()
------------------------------------------------
connect / as sysdba
create or replace directory imagedir as '/home/alamb/quickstart';
-- For windows:
--create or replace directory imagedir as 'c:\quickstart';
grant read on directory imagedir to scott;

call dbms_java.grant_permission('SCOTT','java.io.FilePermission','/home/alamb/quickstart/*','WRITE');
call dbms_java.grant_permission('ORDSYS','java.io.FilePermission','/home/alamb/quickstart/*','WRITE');
-- For windows:
--call dbms_java.grant_permission('SCOTT','java.io.FilePermission','c:\quickstart\*','WRITE');
--call dbms_java.grant_permission('ORDSYS','java.io.FilePermission','c:\quickstart\*','WRITE');

connect scott/tiger
-- Writes the image data from ORDImage with id=<source_id> in image_table
-- to the file named <filename> in the IMAGEDIR directory
create or replace procedure image_export (source_id number, filename varchar2) as
  imgSrc ordsys.ordimage;
  ctx raw(64) := null;
begin
  select image into imgSrc from image_table where id = source_id;
  imgSrc.export(ctx, 'FILE', 'IMAGEDIR', filename);
end;
/

call image_export(3, 'flowers_thumbnail.jpg');
call image_export(4, 'goats_grayscale.jpg');

------------------------------------------------
-- APPENDIX: Using interMedia with Non-Natively 
-- Supported Image Formats
------------------------------------------------
connect scott/tiger
-- imports the image filename from IMAGEDIR in a non natively supported format 
-- into the database. The caller must specify the image's file type, 
-- height, width and mime-type manually via the procedure's parameters
create or replace procedure image_import_other
  (dest_id number, filename varchar2, imgFileFormat varchar2, 
   imgHeight number, imgWidth number, imgMimeType varchar2) is
  img ordsys.ordimage;
  ctx raw(64) := null;
begin
  delete from image_table where id = dest_id;
  insert into image_table (id, image) 
    values (dest_id, ordsys.ordimage.init());
   
  -- import the actual image data into the database and set the 
  -- other ORDImage fields manually. Note that setting the filetype
  -- to 'OTHER:filetype' causes no auto-setting of fields
  select image into img from image_table where id=dest_id for update;
  img.fileFormat := 'OTHER:' || imgFileFormat;
  img.height   := imgHeight;
  img.width    := imgWidth;
  img.mimeType := imgMimeType;
  img.importFrom(ctx, 'file', 'IMAGEDIR', filename);
  img.contentLength := dbms_lob.getlength(img.source.localdata);
  update image_table set image=img where id=dest_id;
end;
/

call image_import_other(6,'dummy.dcm', 'DICOM', 100, 200, 'application/dicom');

-- view the properties we just created (nicely formatted)
column "mimetype" format A20;
select id                         "id", 
       t.image.getWidth()         "width", 
       t.image.getHeight()        "height", 
       t.image.getMimeType()      "mimetype", 
       t.image.getContentLength() "length"
from image_table t;

-- Cleanup from the appendix code
connect scott/tiger
drop procedure image_import_other;


------------------------------------------------
-- Cleaning Up
------------------------------------------------
connect / as sysdba
drop directory imagedir;
call dbms_java.revoke_permission('SCOTT','java.io.FilePermission','/home/alamb/quickstart/*','WRITE');
call dbms_java.revoke_permission('ORDSYS','java.io.FilePermission','/home/alamb/quickstart/*','WRITE');

-- For windows:
--call dbms_java.revoke_permission('SCOTT','java.io.FilePermission', 'c:\quickstart\*','WRITE');
--call dbms_java.revoke_permission('ORDSYS','java.io.FilePermission','c:\quickstart\*','WRITE');

connect scott/tiger
drop procedure image_import;
drop procedure image_processcopy;
drop procedure image_export;
drop table image_table;
